`timescale 1ns / 10ps

module cmos_g (
    input   DATA     , // data input
    input   NCONTROL , // Nch control input
    input   PCONTROL , // Pch control input
    output  OUT        // data output
);

    cmos ucmos ( OUT, DATA, NCONTROL, PCONTROL );

endmodule


module cmos_r (
    input   DATA     , // data input
    input   NCONTROL , // Nch control input
    input   PCONTROL , // Pch control input
    output  OUT        // data output
);

    nmos unmos ( OUT, DATA, NCONTROL );
    pmos upmos ( OUT, DATA, PCONTROL );

endmodule


module tb_cmos ;

reg   DATA ;          // DATA
reg   CNTn , CNTp ;   // NCONTROL, PCONTROL
wire  OUTg , OUTr ;   // OUT Golden, Revise

parameter RATE = 10 ;
integer   fd ;        // file descriptor

    initial begin
        TEST  (1'b0, 1'b0); // Off On
        TEST  (1'b0, 1'b1); // Off Off
        TEST  (1'b0, 1'bx); // Off X
        TEST  (1'b0, 1'bz); // Off Z
        TEST  (1'b1, 1'b0); // On  On
        TEST  (1'b1, 1'bx); // On  X
        TEST  (1'b1, 1'bz); // On  Z
        TEST  (1'bx, 1'bx); // X   X
        TEST  (1'bx, 1'bz); // X   Z
        TEST  (1'bz, 1'bz); // Z   Z
    end

    task TEST ;
        input INn, INp ;
        begin
            CNTn = INn  ;
            CNTp = INp  ;
            DATA = 1'b0 ;
            #(RATE) DATA = 1'b1 ;
            #(RATE) DATA = 1'bx ;
            #(RATE) DATA = 1'bz ;
            #(RATE) ;
        end
    endtask

    // golden : cmos
    cmos_g ucmos_g (
        .DATA     (DATA),
        .NCONTROL (CNTn),
        .PCONTROL (CNTp),
        .OUT      (OUTg)
    );

    // revise : nmos+pmos
    cmos_r ucmos_r (
        .DATA     (DATA),
        .NCONTROL (CNTn),
        .PCONTROL (CNTp),
        .OUT      (OUTr)
    );

    initial begin
        fd = $fopen("tb_cmos.log");
        if (fd == 0) $fclose(fd);

        $fdisplay(fd, $time, " Cn Cp Dt | OUTg    OUTr");
    end

    initial begin
        $fmonitor(fd, $time, "  %d  %d  %d | %d (%v) %d (%v)",
            CNTn, CNTp, DATA, OUTg, OUTg, OUTr, OUTr);
    end


endmodule